package com.ruoyi.system.service.impl;

import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.core.utils.SpringUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.datascope.annotation.DataScope;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysDept;
import com.ruoyi.system.api.domain.SysRole;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.domain.vo.TreeSelect;
import com.ruoyi.system.mapper.SysDeptMapper;
import com.ruoyi.system.mapper.SysRoleMapper;
import com.ruoyi.system.service.ISysDeptService;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class SysDeptServiceImpl implements ISysDeptService {
   @Autowired
   private SysDeptMapper deptMapper;
   @Autowired
   private SysRoleMapper roleMapper;

   @DataScope(
      deptAlias = "d"
   )
   public List<SysDept> selectDeptList(SysDept dept) {
      return this.deptMapper.selectDeptList(dept);
   }

   public List<TreeSelect> selectDeptTreeList(SysDept dept) {
      List<SysDept> depts = ((SysDeptServiceImpl)SpringUtils.getAopProxy(this)).selectDeptList(dept);
      return this.buildDeptTreeSelect(depts);
   }

   public List<SysDept> buildDeptTree(List<SysDept> depts) {
      List<SysDept> returnList = new ArrayList();
      List<Long> tempList = (List)depts.stream().map(SysDept::getDeptId).collect(Collectors.toList());
      Iterator var4 = depts.iterator();

      while(var4.hasNext()) {
         SysDept dept = (SysDept)var4.next();
         if (!tempList.contains(dept.getParentId())) {
            this.recursionFn(depts, dept);
            ((List)returnList).add(dept);
         }
      }

      if (((List)returnList).isEmpty()) {
         returnList = depts;
      }

      return (List)returnList;
   }

   public List<TreeSelect> buildDeptTreeSelect(List<SysDept> depts) {
      List<SysDept> deptTrees = this.buildDeptTree(depts);
      return (List)deptTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
   }

   public List<Long> selectDeptListByRoleId(Long roleId) {
      SysRole role = this.roleMapper.selectRoleById(roleId);
      return this.deptMapper.selectDeptListByRoleId(roleId, role.isDeptCheckStrictly());
   }

   public SysDept selectDeptById(Long deptId) {
      return this.deptMapper.selectDeptById(deptId);
   }

   public int selectNormalChildrenDeptById(Long deptId) {
      return this.deptMapper.selectNormalChildrenDeptById(deptId);
   }

   public boolean hasChildByDeptId(Long deptId) {
      int result = this.deptMapper.hasChildByDeptId(deptId);
      return result > 0;
   }

   public boolean checkDeptExistUser(Long deptId) {
      int result = this.deptMapper.checkDeptExistUser(deptId);
      return result > 0;
   }

   public boolean checkDeptNameUnique(SysDept dept) {
      Long deptId = StringUtils.isNull(dept.getDeptId()) ? -1L : dept.getDeptId();
      SysDept info = this.deptMapper.checkDeptNameUnique(dept.getDeptName(), dept.getParentId());
      return !StringUtils.isNotNull(info) || info.getDeptId() == deptId;
   }

   public void checkDeptDataScope(Long deptId) {
      if (!SysUser.isAdmin(SecurityUtils.getUserId()) && StringUtils.isNotNull(deptId)) {
         SysDept dept = new SysDept();
         dept.setDeptId(deptId);
         List<SysDept> depts = ((SysDeptServiceImpl)SpringUtils.getAopProxy(this)).selectDeptList(dept);
         if (StringUtils.isEmpty(depts)) {
            throw new ServiceException("没有权限访问部门数据！");
         }
      }

   }

   public int insertDept(SysDept dept) {
      SysDept info = this.deptMapper.selectDeptById(dept.getParentId());
      if (!"0".equals(info.getStatus())) {
         throw new ServiceException("部门停用，不允许新增");
      } else {
         dept.setAncestors(info.getAncestors() + "," + dept.getParentId());
         return this.deptMapper.insertDept(dept);
      }
   }

   public int updateDept(SysDept dept) {
      SysDept newParentDept = this.deptMapper.selectDeptById(dept.getParentId());
      SysDept oldDept = this.deptMapper.selectDeptById(dept.getDeptId());
      if (StringUtils.isNotNull(newParentDept) && StringUtils.isNotNull(oldDept)) {
         String newAncestors = newParentDept.getAncestors() + "," + newParentDept.getDeptId();
         String oldAncestors = oldDept.getAncestors();
         dept.setAncestors(newAncestors);
         this.updateDeptChildren(dept.getDeptId(), newAncestors, oldAncestors);
      }

      int result = this.deptMapper.updateDept(dept);
      if ("0".equals(dept.getStatus()) && StringUtils.isNotEmpty(dept.getAncestors()) && !StringUtils.equals("0", dept.getAncestors())) {
         this.updateParentDeptStatusNormal(dept);
      }

      return result;
   }

   private void updateParentDeptStatusNormal(SysDept dept) {
      String ancestors = dept.getAncestors();
      Long[] deptIds = Convert.toLongArray(ancestors);
      this.deptMapper.updateDeptStatusNormal(deptIds);
   }

   public void updateDeptChildren(Long deptId, String newAncestors, String oldAncestors) {
      List<SysDept> children = this.deptMapper.selectChildrenDeptById(deptId);
      Iterator var5 = children.iterator();

      while(var5.hasNext()) {
         SysDept child = (SysDept)var5.next();
         child.setAncestors(child.getAncestors().replaceFirst(oldAncestors, newAncestors));
      }

      if (children.size() > 0) {
         this.deptMapper.updateDeptChildren(children);
      }

   }

   public int deleteDeptById(Long deptId) {
      return this.deptMapper.deleteDeptById(deptId);
   }

   private void recursionFn(List<SysDept> list, SysDept t) {
      List<SysDept> childList = this.getChildList(list, t);
      t.setChildren(childList);
      Iterator var4 = childList.iterator();

      while(var4.hasNext()) {
         SysDept tChild = (SysDept)var4.next();
         if (this.hasChild(list, tChild)) {
            this.recursionFn(list, tChild);
         }
      }

   }

   private List<SysDept> getChildList(List<SysDept> list, SysDept t) {
      List<SysDept> tlist = new ArrayList();
      Iterator<SysDept> it = list.iterator();

      while(it.hasNext()) {
         SysDept n = (SysDept)it.next();
         if (StringUtils.isNotNull(n.getParentId()) && n.getParentId() == t.getDeptId()) {
            tlist.add(n);
         }
      }

      return tlist;
   }

   private boolean hasChild(List<SysDept> list, SysDept t) {
      return this.getChildList(list, t).size() > 0;
   }
}
